<?php
/**
 * @package		Joomla
 * @subpackage	JTags
 * @copyright	Copyright (C) 2005 - 2008 Open Source Matters. All rights reserved.
 * @license		GNU/GPL, see LICENSE.php
 * Joomla! is free software. This version may have been modified pursuant to the
 * GNU General Public License, and as distributed it includes or is derivative
 * of works licensed under the GNU General Public License or other free or open
 * source software licenses. See COPYRIGHT.php for copyright notices and
 * details.
 */

/* Check to ensure this file is included in Joomla! */
defined( '_JEXEC' ) or die( 'Restricted access' );

jimport( 'joomla.application.component.controller' );

/**
 * JTags Component Controller
 *
 * @author		Jacek Zielinski <jacek.zielinski@community.joomla.org>
 * @package		Joomla
 * @subpackage	JTags
 * @since 1.5
 */

class JTagsController extends JController
{
	function __construct( $default = array() )
	{
		parent::__construct( $default );
		$this->registerTask( 'add' , 'edit' );
		$this->registerTask( 'apply', 'save' );
		$this->registerTask( 'savetags', 'saveTags' );
		$this->registerTask( 'remove', 'removeTags' );
	}

	/**
	* Shows tags
	*
	* @access	public
	* @since	1.5
	*/
	function showTags()
	{
		global $option, $mainframe;

		$limit = JRequest::getVar('limit', 20);
		$limitstart = JRequest::getVar('limitstart', 0);

		$db =& JFactory::getDBO();

		$query = 'SELECT SQL_CALC_FOUND_ROWS #__jtags_tags.tag_id, name, COUNT( #__jtags_tags.tag_id ) AS used, publish_down
				  FROM #__jtags_items
				  NATURAL JOIN #__jtags_tags
				  JOIN #__content ON (#__jtags_items.item_id = #__content.id)
				  WHERE state = 1 AND NOW() > publish_up
				  GROUP BY #__jtags_tags.tag_id
				  ORDER BY name ASC';
		$db->setQuery( $query, $limitstart, $limit );
		$rows = $db->loadObjectList(); 

		if ($db->getErrorNum()) {
			echo $db->stderr();
			return false;
		}
		$db->setQuery('SELECT FOUND_ROWS();');
		$total = $db->loadResult();
		
		jimport('joomla.html.pagination');

		$pageNav = new JPagination($total, $limitstart, $limit);

		HTML_jtags::showTags( $option, $rows, $pageNav );
	}

	/**
	* Edits specified tag
	*
	* @access	public
	* @since	1.5
	*/
	function edit()
	{
		global $option;
		$cid = JRequest::getVar( 'cid' , array(0) , '' , 'array' );
		$tagId = $cid[0];

		$row =& JTable::getInstance('jtags', 'Table');
		$row->load($tagId);

		HTML_jtags::editTag($option, $row);
	}

	/**
	* Saves specified tag
	*
	* @access	public
	* @since	1.5
	*/
	function save()
	{
		global $option;
		$row =& JTable::getInstance('jtags', 'Table');

		if (!$row->bind(JRequest::get('post'))) {
			echo "<script> alert('".$row->getError()."'); window.history.go(-1); </script>\n";
			exit();
		}

		$row->name = JRequest::getVar( 'tagName', '', 'post', 'string', JREQUEST_ALLOWRAW );
		//if save to DB fails
		if (!$row->store()) {
			echo "<script> alert('".$row->getError()."'); window.history.go(-1); </script>\n";
			exit();
		}

		switch ($this->_task)
		{
			case 'apply':
				$msg = JText::_('Tag updated');
				$link = 'index.php?option=' . $option . '&task=edit&cid[]='. $row->tag_id;
				break;

			case 'save':
			default:
				$msg = JText::_('Tag saved');
				$link = 'index.php?option=' . $option;
				break;
		}

		$this->setRedirect($link, $msg);
	}

	/**
	* Removes all tags not linked to any article. Just to keep database clean.
	*
	* @access	public
	* @since	1.5
	*/
	function removeTags()
	{
		global $option;

		$db =& JFactory::getDBO();

		$query = 'DELETE FROM #__jtags_items
				  USING #__jtags_items
				  NATURAL JOIN #__jtags_tags
				  LEFT JOIN #__content
				  ON (#__jtags_items.item_id = #__content.id)
				  WHERE (state IS NULL)';
	    $db->setQuery( $query );
	    if (!$db->query())
	    {
	      echo "<script> alert('".$db->getErrorMsg()."');
	      window.history.go(-1); </script>\n";
	    }

		$aRows = $db->getAffectedRows();

		$query = 'DELETE FROM #__jtags_tags
				  USING #__jtags_tags
				  NATURAL LEFT JOIN #__jtags_items
				  WHERE item_id IS NULL';
	    $db->setQuery( $query );
	    if (!$db->query())
	    {
	      echo "<script> alert('".$db->getErrorMsg()."');
	      window.history.go(-1); </script>\n";
	    }

		$bRows = $db->getAffectedRows();

		$affectedRows = $aRows + $bRows;

		$link = 'index.php?option=' . $option;
		$this->setRedirect($link, 'Useless entries ('.$affectedRows.' rows) has been successfully removed.');
	}
}

?>