<?php
/**
* @version		$Id: language.php 0002 2008-12-12 00:00:00Z yvolk $
* @package		yvTransliteratePlugin
* @copyright	2007-2008 yvolk (Yuri Volkov), http://yurivolkov.com. All rights reserved.
* @license GPL
*/

// Check to ensure this file is within the rest of the framework
defined('JPATH_BASE') or die();

require_once (JPATH_SITE.DS.'libraries'.DS.'joomla'.DS.'language'.DS.'language.php');

/**
 * Languages/translation handler class
 */
class yvLanguage extends JObject
{
	var $_debug = false;

	/**
	 * Array holding the language metadata
	 *
	 * @var array
	 * @access protected
	 */
	var $_metadata 	= null;
	var $_HasTransliteration = false;

	/**
	 * The language tag (Identifying string of the language)
	 *
	 * @var string
	 * @access protected
	 */
	var $_langTag = null;

	/**
	* Constructor activating the default information of the language
	*
	* @access protected
	*/
	function __construct($langTag)
	{
		$this->setLanguage($langTag);
	}

	/**
	* Get for the language tag (as defined in RFC 3066)
	*
	* @access	public
	* @return	string The language tag
	* @since	1.5
	*/
	function getTag() {
		return $this->_langTag;
	}

	/**
	 * Set the language attributes to the given language
	 *
	 * Once called, the language still needs to be loaded using yvLanguage::load()
	 *
	 * @access	public
	 * @param	string	$langTag	Language tag
	 */
	function setLanguage($langTag)
	{
		$this->_langTag		= $langTag;
		$this->_metadata	= $this->getMetadata($this->_langTag);

	}

	function HasTransliteration() {
		return $this->_HasTransliteration;
	}

	/**
	 * Returns a associative array holding the metadata
	 *
	 * @access public
	 * @param string	The tag of the language
	 * @return mixed	If $langTag exists return key/value pair with the language metadata,
	 *  				otherwise return NULL
	 */

	function getMetadata($langTag)
	{
		$path = JLanguage::getLanguagePath(JPATH_ADMINISTRATOR, $langTag);
		$file = $langTag.'.yvtransliterate.xml';

		$result = null;
		$fullname = $path.DS.$file;
		$found = is_file($fullname);
		if ($this->_debug) {
			echo 'langTag="' . $langTag . '", file="' . $fullname . '" ' . ($found ? 'found' : 'not found') . '<br/>';
		}
		if($found) {
			$result = yvLanguage::_parseXMLLanguageFile($fullname);
		}

		return $result;
	}

	/**
	 * Parse XML file for language information
	 *
	 * @access public
	 * @param string	$path	 Path to the xml files
	 * @return array	Array holding the found metadata as a key => value pair
	 */
	function _parseXMLLanguageFile($path)
	{
		jimport('joomla.factory');
		$xml = & JFactory::getXMLParser('Simple');
		if (!$xml->loadFile($path)) {
			return null;
		}

		// Check that it's am metadata file
		if ($xml->document->name() != 'metafile') {
			return null;
		}

		$metadata = array ();

		//if ($xml->document->attributes('type') == 'language') {

			foreach ($xml->document->metadata[0]->children() as $child) {
			  switch ($child->name()) {
			    case 'transliterations' :
			      $transliterations = array();
			      foreach ($child->children() as $transliterationMetadata) {
			        $transliteration = new yvTransliteration($transliterationMetadata);
			        //echo "transliteration->getName=" . $transliteration->getName();
			        $transliterations[$transliteration->getName()] = $transliteration;
			        $this->_HasTransliteration = true;
			      }
   				  $metadata[$child->name()] = $transliterations;
			      break;
			    default :
    				$metadata[$child->name()] = $child->data();
			  }
			}
		//}
		return $metadata;
	}
	
	/**
	 * Transliterate string
	 *
	 * @access public
	 * @param string	$StringToTransliterate
	 * @param string  $DestinationLangTag
	 * @param string  $TransliterationMethodName
	 * @return string	Transliterated string
	 */
  function transliterate( $StringToTransliterate, $DestinationLangTag = 'en-GB', $TransliterationMethodName = 'default')
  {
  	if (!$this->_HasTransliteration) {
      // echo '(no transliterations)<br />';
      // return input string without changes
      return $StringToTransliterate;
  	}
    if ($this->_langTag == $DestinationLangTag) {
      // echo '(nothing to do)<br />';
      // return input string without changes
      return $StringToTransliterate;
    }
    
    $transliteration = null;
    $sameLanguage = null; // 
    // find yvTransliteration object
    foreach ($this->_metadata['transliterations'] as $trans) {
      // echo '(found: ' . $trans->getDestinationLangTag() . '; needed: ' . $DestinationLangTag . ')<br />';
      if ($trans->getDestinationLangTag() == $DestinationLangTag) {
        if (!$sameLanguage) {
          $sameLanguage = $trans;
        }
        if ($TransliterationMethodName == 'default' ) {
          // echo '(first transliteration is default)<br />';
          $transliteration = $trans;
          break;
        }
        if ($trans->getName() == $DestinationLangTag) {
          // echo '(exact match found)<br />';
          $transliteration = $trans;
          break;
        }
      }
    }

    if ($transliteration) {
      return $transliteration->transliterate($StringToTransliterate);
    } else {
      // echo '(return input string without changes)<br />';
      return $StringToTransliterate;
    }
  }

	/**
	* Set the Debug property
	*
	* @access public
	*/
	function setDebug($debug) {
		$this->_debug = $debug;
	}

	/**
	* Get the Debug property
	*
	* @access public
	* @return boolean True is in debug mode
	*/
	function getDebug() {
		return $this->_debug;
	}

}  // yvLanguage Class
	
/**
 * Transliteration class
 * Private class for yvLanguage class, 
 *   yvLanguage class holds information about Source Language of The transliteration.
 *
 * @author		Yuri Volkov <yvolk@yurivolkov.com>
 */
Class yvTransliteration extends JObject
{
	var $_debug = false;

  // name of the Transliteration
  var $_name = null;
  
  var $_description	= null;
	
	/**
	 * The language tag (Identifying string of the language)
	 *
	 * @var string
	 * @access private
	 */
	var $_destinationLangTag = null;

	/**
	 * Transliteration table. 
	 * Two arrays of strings. Both arrays are of the same size.
	 *
	 * @var array()
	 * @access private
	 */
  var $_tableFrom = array();
  var $_tableTo = array();

  /**
  * Constructor loading transliteration metadata
  * is supposed to be called from yvLanguage Class
  * @access protected
  */
  function __construct($metadata)
  {

		if ($this->_debug) {
			echo 'yvTransliteration::__construct, metadata:<br/>';
	    foreach ($metadata->children() as $row) {
	    	echo "&nbsp;&nbsp;Name: " . $row->name() . "; Data:'" . $row->data() . "'<br />";
	    }
			//echo '<hr/><pre>' . print_r($metadata, true) . '</pre><hr/>';
		}

    $this->_destinationLangTag = $metadata->destinationLangTag[0]->data();
    // echo $this->_destinationLangTag;
    // echo " point 2 <br />";

    $this->_name = $metadata->name[0]->data();
    $this->_description = $metadata->description[0]->data();
    
    $table = array();
    foreach ($metadata->table[0]->children() as $row) {
      $from = $row->from[0]->data();
      if (strlen($from) > 0) {
        $this->_tableFrom[] = $from;
	      $this->_tableTo[] = $row->to[0]->data();
      }
    }
  }

  function getDestinationLangTag() {
    return $this->_destinationLangTag;
  }

  function getName() {
    return $this->_name;
  }

  function getDescription() {
    return $this->_description;
  }

	/**
	 * Transliterate string
	 *
	 * @access protected (from yvLanguage class only)
	 * @param string	$StringToTransliterate
	 * @return string	Transliterated string
	 */
  function transliterate( $StringToTransliterate)
  {
    $pos = 0;
    $pos_end = strlen($StringToTransliterate); // ??maybe JString::strlen...
    $strOut = "";
    
    while ($pos < $pos_end) {
      // find substitution for the substring, starting with $pos position
      $found = false;
      for ($ind = 0; $ind < count($this->_tableFrom); $ind++) { 
        $str1 = $this->_tableFrom[$ind];
        $len1 = strlen($str1);
		    if (substr($StringToTransliterate, $pos, $len1) == $str1) {
		      // case sensitive match found
		      $found = true;
          $strOut .= $this->_tableTo[$ind];
		      $pos += $len1;
		      break;
		    }
      }
      if (!$found) {
        // copy one char from input to output and move one char forward
        $strOut .= substr($StringToTransliterate, $pos, 1);
	      $pos += 1;
      }
    }  // while

    return $strOut;
  }

} // Class yvTransliteration
?>
